'''
Neuron simulator export for:

Components:
    null (Type: notes)
    i_clamp (Type: pulseGenerator:  delay=0.2 (SI time) duration=0.6 (SI time) amplitude=9.9E-10 (SI current))
    testcell (Type: IF_cond_alpha:  e_rev_E=0.0 (dimensionless) e_rev_I=-70.0 (dimensionless) tau_refrac=5.0 (dimensionless) v_thresh=-50.0 (dimensionless) tau_m=20.0 (dimensionless) v_rest=-65.0 (dimensionless) v_reset=-65.0 (dimensionless) cm=1.0 (dimensionless) i_offset=0.1 (dimensionless) tau_syn_E=0.3 (dimensionless) tau_syn_I=0.5 (dimensionless) v_init=-65.0 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NFARAD=1.0E-9 (SI capacitance))
    testcell2 (Type: IF_cond_alpha:  e_rev_E=-10.0 (dimensionless) e_rev_I=-80.0 (dimensionless) tau_refrac=5.0 (dimensionless) v_thresh=-50.0 (dimensionless) tau_m=20.0 (dimensionless) v_rest=-65.0 (dimensionless) v_reset=-65.0 (dimensionless) cm=1.0 (dimensionless) i_offset=-0.1 (dimensionless) tau_syn_E=2.0 (dimensionless) tau_syn_I=10.0 (dimensionless) v_init=-65.0 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NFARAD=1.0E-9 (SI capacitance))
    ampaSyn (Type: alphaCondSynapse:  e_rev=-10.0 (dimensionless) tau_syn=2.0 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NAMP=1.0E-9 (SI current))
    gabaSyn (Type: alphaCondSynapse:  e_rev=-80.0 (dimensionless) tau_syn=10.0 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NAMP=1.0E-9 (SI current))
    Example4_PyNN (Type: network)
    SimExample4 (Type: Simulation:  length=1.0 (SI time) step=1.0E-5 (SI time))


    This NEURON file has been generated by org.neuroml.export (see https://github.com/NeuroML/org.neuroml.export)
         org.neuroml.export  v1.8.1
         org.neuroml.model   v1.8.1
         jLEMS               v0.10.6

'''

import neuron

import time
import datetime
import sys

import hashlib
h = neuron.h
h.load_file("stdlib.hoc")

h.load_file("stdgui.hoc")

h("objref p")
h("p = new PythonObject()")

class NeuronSimulation():

    def __init__(self, tstop, dt, seed=5678):

        print("\n    Starting simulation in NEURON of %sms generated from NeuroML2 model...\n"%tstop)

        self.setup_start = time.time()
        self.seed = seed
        import socket
        self.report_file = open('report.SimExample4.txt','w')
        print('Simulator version:  %s'%h.nrnversion())
        self.report_file.write('# Report of running simulation with %s\n'%h.nrnversion())
        self.report_file.write('Simulator=NEURON\n')
        self.report_file.write('SimulatorVersion=%s\n'%h.nrnversion())

        self.report_file.write('SimulationFile=%s\n'%__file__)
        self.report_file.write('PythonVersion=%s\n'%sys.version.replace('\n',' '))
        print('Python version:     %s'%sys.version.replace('\n',' '))
        self.report_file.write('NeuroMLExportVersion=1.8.1\n')
        self.report_file.write('SimulationSeed=%s\n'%self.seed)
        self.report_file.write('Hostname=%s\n'%socket.gethostname())
        self.randoms = []
        self.next_global_id = 0  # Used in Random123 classes for elements using random(), etc. 

        self.next_spiking_input_id = 0  # Used in Random123 classes for elements using random(), etc. 

        '''
        Adding simulation Component(id=SimExample4 type=Simulation) of network/component: Example4_PyNN (Type: network)
        
        '''
        # ######################   Population: pop0
        print("Population pop0 contains 2 instance(s) of component: testcell of type: IF_cond_alpha")

        h(" {n_pop0 = 2} ")
        '''
        Population pop0 contains instances of Component(id=testcell type=IF_cond_alpha)
        whose dynamics will be implemented as a mechanism (testcell) in a mod file
        '''
        h(" create pop0[2]")
        h(" objectvar m_testcell_pop0[2] ")

        for i in range(int(h.n_pop0)):
            h.pop0[i].L = 10.0
            h.pop0[i](0.5).diam = 10.0
            h.pop0[i](0.5).cm = 318.3098861837907
            h.pop0[i].push()
            h(" pop0[%i]  { m_testcell_pop0[%i] = new testcell(0.5) } "%(i,i))

            h.m_testcell_pop0[i].e_rev_E = 0.0
            h.m_testcell_pop0[i].e_rev_I = -70.0
            h.m_testcell_pop0[i].tau_refrac = 5.0
            h.m_testcell_pop0[i].v_thresh = -50.0
            h.m_testcell_pop0[i].tau_m = 20.0
            h.m_testcell_pop0[i].v_rest = -65.0
            h.m_testcell_pop0[i].v_reset = -65.0
            h.m_testcell_pop0[i].cm = 1.0
            h.m_testcell_pop0[i].i_offset = 0.1
            h.m_testcell_pop0[i].tau_syn_E = 0.3
            h.m_testcell_pop0[i].tau_syn_I = 0.5
            h.m_testcell_pop0[i].v_init = -65.0
            h.m_testcell_pop0[i].MSEC = 1.0
            h.m_testcell_pop0[i].MVOLT = 1.0
            h.m_testcell_pop0[i].NFARAD = 0.001
            h.pop_section()

            self.next_global_id+=1

        h(" pop0[0] { pt3dclear() } ")
        h(" pop0[0] { pt3dadd(966.45355, 44.07326 + (5), 7.49147, 10) } ")
        h(" pop0[0] { pt3dadd(966.45355, 44.07326 + (-5), 7.49147, 10) } ")
        h(" pop0[1] { pt3dclear() } ")
        h(" pop0[1] { pt3dadd(910.97595, 93.9269 + (5), 582.2276, 10) } ")
        h(" pop0[1] { pt3dadd(910.97595, 93.9269 + (-5), 582.2276, 10) } ")

        # ######################   Population: pop1
        print("Population pop1 contains 2 instance(s) of component: testcell2 of type: IF_cond_alpha")

        h(" {n_pop1 = 2} ")
        '''
        Population pop1 contains instances of Component(id=testcell2 type=IF_cond_alpha)
        whose dynamics will be implemented as a mechanism (testcell2) in a mod file
        '''
        h(" create pop1[2]")
        h(" objectvar m_testcell2_pop1[2] ")

        for i in range(int(h.n_pop1)):
            h.pop1[i].L = 10.0
            h.pop1[i](0.5).diam = 10.0
            h.pop1[i](0.5).cm = 318.3098861837907
            h.pop1[i].push()
            h(" pop1[%i]  { m_testcell2_pop1[%i] = new testcell2(0.5) } "%(i,i))

            h.m_testcell2_pop1[i].e_rev_E = -10.0
            h.m_testcell2_pop1[i].e_rev_I = -80.0
            h.m_testcell2_pop1[i].tau_refrac = 5.0
            h.m_testcell2_pop1[i].v_thresh = -50.0
            h.m_testcell2_pop1[i].tau_m = 20.0
            h.m_testcell2_pop1[i].v_rest = -65.0
            h.m_testcell2_pop1[i].v_reset = -65.0
            h.m_testcell2_pop1[i].cm = 1.0
            h.m_testcell2_pop1[i].i_offset = -0.1
            h.m_testcell2_pop1[i].tau_syn_E = 2.0
            h.m_testcell2_pop1[i].tau_syn_I = 10.0
            h.m_testcell2_pop1[i].v_init = -65.0
            h.m_testcell2_pop1[i].MSEC = 1.0
            h.m_testcell2_pop1[i].MVOLT = 1.0
            h.m_testcell2_pop1[i].NFARAD = 0.001
            h.pop_section()

            self.next_global_id+=1

        h(" pop1[0] { pt3dclear() } ")
        h(" pop1[0] { pt3dadd(671.5635, 8.393823 + (5), 766.48096, 10) } ")
        h(" pop1[0] { pt3dadd(671.5635, 8.393823 + (-5), 766.48096, 10) } ")
        h(" pop1[1] { pt3dclear() } ")
        h(" pop1[1] { pt3dadd(236.80977, 3.081402 + (5), 788.7727, 10) } ")
        h(" pop1[1] { pt3dadd(236.80977, 3.081402 + (-5), 788.7727, 10) } ")

        # ######################   Population: pop2
        print("Population pop2 contains 1 instance(s) of component: testcell2 of type: IF_cond_alpha")

        h(" {n_pop2 = 1} ")
        '''
        Population pop2 contains instances of Component(id=testcell2 type=IF_cond_alpha)
        whose dynamics will be implemented as a mechanism (testcell2) in a mod file
        '''
        h(" create pop2[1]")
        h(" objectvar m_testcell2_pop2[1] ")

        for i in range(int(h.n_pop2)):
            h.pop2[i].L = 10.0
            h.pop2[i](0.5).diam = 10.0
            h.pop2[i](0.5).cm = 318.3098861837907
            h.pop2[i].push()
            h(" pop2[%i]  { m_testcell2_pop2[%i] = new testcell2(0.5) } "%(i,i))

            h.m_testcell2_pop2[i].e_rev_E = -10.0
            h.m_testcell2_pop2[i].e_rev_I = -80.0
            h.m_testcell2_pop2[i].tau_refrac = 5.0
            h.m_testcell2_pop2[i].v_thresh = -50.0
            h.m_testcell2_pop2[i].tau_m = 20.0
            h.m_testcell2_pop2[i].v_rest = -65.0
            h.m_testcell2_pop2[i].v_reset = -65.0
            h.m_testcell2_pop2[i].cm = 1.0
            h.m_testcell2_pop2[i].i_offset = -0.1
            h.m_testcell2_pop2[i].tau_syn_E = 2.0
            h.m_testcell2_pop2[i].tau_syn_I = 10.0
            h.m_testcell2_pop2[i].v_init = -65.0
            h.m_testcell2_pop2[i].MSEC = 1.0
            h.m_testcell2_pop2[i].MVOLT = 1.0
            h.m_testcell2_pop2[i].NFARAD = 0.001
            h.pop_section()

            self.next_global_id+=1

        h(" pop2[0] { pt3dclear() } ")
        h(" pop2[0] { pt3dadd(346.08896, 62.328148 + (5), 615.8157, 10) } ")
        h(" pop2[0] { pt3dadd(346.08896, 62.328148 + (-5), 615.8157, 10) } ")

        # ######################   Projection: proj0
        print("Adding projection: proj0, from pop0 to pop1 with synapse ampaSyn, 4 connection(s)")

        h("objectvar syn_proj0_ampaSyn[4]")

        h("objectvar netConn_proj0_ampaSyn[4]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on pop0[0]] -> cell 0, seg 0 (0.5) [0.5 on pop1[0]], weight: 0.02, delay 2.0
        h("pop1[0] syn_proj0_ampaSyn[0] = new ampaSyn(0.5)")
        h("pop0[0] netConn_proj0_ampaSyn[0] = new NetCon(&v(0.5), syn_proj0_ampaSyn[0], -50.0, 2.0, 0.02)")  

        # Connection 1: cell 0, seg 0 (0.5) [0.5 on pop0[0]] -> cell 1, seg 0 (0.5) [0.5 on pop1[1]], weight: 0.02, delay 2.0
        h("pop1[1] syn_proj0_ampaSyn[1] = new ampaSyn(0.5)")
        h("pop0[0] netConn_proj0_ampaSyn[1] = new NetCon(&v(0.5), syn_proj0_ampaSyn[1], -50.0, 2.0, 0.02)")  

        # Connection 2: cell 1, seg 0 (0.5) [0.5 on pop0[1]] -> cell 0, seg 0 (0.5) [0.5 on pop1[0]], weight: 0.02, delay 2.0
        h("pop1[0] syn_proj0_ampaSyn[2] = new ampaSyn(0.5)")
        h("pop0[1] netConn_proj0_ampaSyn[2] = new NetCon(&v(0.5), syn_proj0_ampaSyn[2], -50.0, 2.0, 0.02)")  

        # Connection 3: cell 1, seg 0 (0.5) [0.5 on pop0[1]] -> cell 1, seg 0 (0.5) [0.5 on pop1[1]], weight: 0.02, delay 2.0
        h("pop1[1] syn_proj0_ampaSyn[3] = new ampaSyn(0.5)")
        h("pop0[1] netConn_proj0_ampaSyn[3] = new NetCon(&v(0.5), syn_proj0_ampaSyn[3], -50.0, 2.0, 0.02)")  

        # ######################   Projection: proj1
        print("Adding projection: proj1, from pop0 to pop2 with synapse gabaSyn, 2 connection(s)")

        h("objectvar syn_proj1_gabaSyn[2]")

        h("objectvar netConn_proj1_gabaSyn[2]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on pop0[0]] -> cell 0, seg 0 (0.5) [0.5 on pop2[0]], weight: 0.01, delay 2.0
        h("pop2[0] syn_proj1_gabaSyn[0] = new gabaSyn(0.5)")
        h("pop0[0] netConn_proj1_gabaSyn[0] = new NetCon(&v(0.5), syn_proj1_gabaSyn[0], -50.0, 2.0, 0.01)")  

        # Connection 1: cell 1, seg 0 (0.5) [0.5 on pop0[1]] -> cell 0, seg 0 (0.5) [0.5 on pop2[0]], weight: 0.01, delay 2.0
        h("pop2[0] syn_proj1_gabaSyn[1] = new gabaSyn(0.5)")
        h("pop0[1] netConn_proj1_gabaSyn[1] = new NetCon(&v(0.5), syn_proj1_gabaSyn[1], -50.0, 2.0, 0.01)")  

        print("Processing 1 input lists")

        # ######################   Input List: stim
        # Adding single input: Component(id=0 type=input)
        h("objref stim_0")
        h("pop0[0] { stim_0 = new i_clamp(0.5) } ")

        print("Finished processing 1 input lists")

        trec = h.Vector()
        trec.record(h._ref_t)

        h.tstop = tstop

        h.dt = dt

        h.steps_per_ms = 1/h.dt



        # ######################   File to save: SimExample4.pop2.v.dat (SimExample4_pop2_v_dat)
        # Column: pop2/0/testcell2/v
        h(' objectvar v_pop2_0_testcell2_v_SimExample4_pop2_v_dat ')
        h(' { v_pop2_0_testcell2_v_SimExample4_pop2_v_dat = new Vector() } ')
        h(' { v_pop2_0_testcell2_v_SimExample4_pop2_v_dat.record(&pop2[0].v(0.5)) } ')
        h.v_pop2_0_testcell2_v_SimExample4_pop2_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample4.pop0.spikes (SimExample4_pop0_spikes)
        h(' objectvar spiketimes_SimExample4_pop0_spikes, t_spiketimes_SimExample4_pop0_spikes ')
        h(' { spiketimes_SimExample4_pop0_spikes = new Vector() } ')
        h(' { t_spiketimes_SimExample4_pop0_spikes = new Vector() } ')
        h(' objref netConnSpike_SimExample4_pop0_spikes, nil ')
        # Column: pop0/0/testcell (0) pop0[0]
        h(' pop0[0] { netConnSpike_SimExample4_pop0_spikes = new NetCon(&v(0.5), nil, -50.0, 0, 1) } ')
        h(' { netConnSpike_SimExample4_pop0_spikes.record(t_spiketimes_SimExample4_pop0_spikes, spiketimes_SimExample4_pop0_spikes, 0) } ')
        # Column: pop0/1/testcell (1) pop0[1]
        h(' pop0[1] { netConnSpike_SimExample4_pop0_spikes = new NetCon(&v(0.5), nil, -50.0, 0, 1) } ')
        h(' { netConnSpike_SimExample4_pop0_spikes.record(t_spiketimes_SimExample4_pop0_spikes, spiketimes_SimExample4_pop0_spikes, 1) } ')

        # ######################   File to save: time.dat (time)
        # Column: time
        h(' objectvar v_time ')
        h(' { v_time = new Vector() } ')
        h(' { v_time.record(&t) } ')
        h.v_time.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample4.pop1.v.dat (SimExample4_pop1_v_dat)
        # Column: pop1/0/testcell2/v
        h(' objectvar v_pop1_0_testcell2_v_SimExample4_pop1_v_dat ')
        h(' { v_pop1_0_testcell2_v_SimExample4_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_0_testcell2_v_SimExample4_pop1_v_dat.record(&pop1[0].v(0.5)) } ')
        h.v_pop1_0_testcell2_v_SimExample4_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/1/testcell2/v
        h(' objectvar v_pop1_1_testcell2_v_SimExample4_pop1_v_dat ')
        h(' { v_pop1_1_testcell2_v_SimExample4_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_1_testcell2_v_SimExample4_pop1_v_dat.record(&pop1[1].v(0.5)) } ')
        h.v_pop1_1_testcell2_v_SimExample4_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample4.pop0.v.dat (SimExample4_pop0_v_dat)
        # Column: pop0/0/testcell/v
        h(' objectvar v_pop0_0_testcell_v_SimExample4_pop0_v_dat ')
        h(' { v_pop0_0_testcell_v_SimExample4_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_0_testcell_v_SimExample4_pop0_v_dat.record(&pop0[0].v(0.5)) } ')
        h.v_pop0_0_testcell_v_SimExample4_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop0/1/testcell/v
        h(' objectvar v_pop0_1_testcell_v_SimExample4_pop0_v_dat ')
        h(' { v_pop0_1_testcell_v_SimExample4_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_1_testcell_v_SimExample4_pop0_v_dat.record(&pop0[1].v(0.5)) } ')
        h.v_pop0_1_testcell_v_SimExample4_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        self.initialized = False

        self.sim_end = -1 # will be overwritten

        setup_end = time.time()
        self.setup_time = setup_end - self.setup_start
        print("Setting up the network to simulate took %f seconds"%(self.setup_time))

    def run(self):

        self.initialized = True
        sim_start = time.time()
        print("Running a simulation of %sms (dt = %sms; seed=%s)" % (h.tstop, h.dt, self.seed))

        try:
            h.run()
        except Exception as e:
            print("Exception running NEURON: %s" % (e))
            quit()


        self.sim_end = time.time()
        self.sim_time = self.sim_end - sim_start
        print("Finished NEURON simulation in %f seconds (%f mins)..."%(self.sim_time, self.sim_time/60.0))

        try:
            self.save_results()
        except Exception as e:
            print("Exception saving results of NEURON simulation: %s" % (e))
            quit()


    def advance(self):

        if not self.initialized:
            h.finitialize()
            self.initialized = True

        h.fadvance()


    ###############################################################################
    # Hash function to use in generation of random value
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _id32 (self,obj): 
        return int(hashlib.md5(obj.encode('utf-8')).hexdigest()[0:8],16)  # convert 8 first chars of md5 hash in base 16 to int


    ###############################################################################
    # Initialize the stim randomizer
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _init_stim_randomizer(self,rand, stimType, gid, seed): 
        #print("INIT STIM  %s; %s; %s; %s"%(rand, stimType, gid, seed))
        rand.Random123(self._id32(stimType), gid, seed)


    def save_results(self):

        print("Saving results at t=%s..."%h.t)

        if self.sim_end < 0: self.sim_end = time.time()


        # ######################   File to save: time.dat (time)
        py_v_time = [ t/1000 for t in h.v_time.to_python() ]  # Convert to Python list for speed...

        f_time_f2 = open('time.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_time_f2.write('%f'% py_v_time[i])  # Save in SI units...
        f_time_f2.close()
        print("Saved data to: time.dat")

        # ######################   File to save: SimExample4.pop2.v.dat (SimExample4_pop2_v_dat)
        py_v_pop2_0_testcell2_v_SimExample4_pop2_v_dat = [ float(x  / 1000.0) for x in h.v_pop2_0_testcell2_v_SimExample4_pop2_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample4_pop2_v_dat_f2 = open('SimExample4.pop2.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample4_pop2_v_dat_f2.write('%e\t%e\t\n' % (py_v_time[i], py_v_pop2_0_testcell2_v_SimExample4_pop2_v_dat[i], ))
        f_SimExample4_pop2_v_dat_f2.close()
        print("Saved data to: SimExample4.pop2.v.dat")

        # ######################   File to save: SimExample4.pop0.spikes (SimExample4_pop0_spikes)

        f_SimExample4_pop0_spikes_f2 = open('SimExample4.pop0.spikes', 'w')
        h(' objref netConnSpike_SimExample4_pop0_spikes ')
        spike_ids = h.spiketimes_SimExample4_pop0_spikes.to_python()  
        spike_times = h.t_spiketimes_SimExample4_pop0_spikes.to_python()
        for i, id in enumerate(spike_ids):
            # Saving in format: ID_TIME
            f_SimExample4_pop0_spikes_f2.write("%i\t%s\n"%(id,spike_times[i]/1000.0))
        f_SimExample4_pop0_spikes_f2.close()
        print("Saved data to: SimExample4.pop0.spikes")

        # ######################   File to save: SimExample4.pop1.v.dat (SimExample4_pop1_v_dat)
        py_v_pop1_0_testcell2_v_SimExample4_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_0_testcell2_v_SimExample4_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_1_testcell2_v_SimExample4_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_1_testcell2_v_SimExample4_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample4_pop1_v_dat_f2 = open('SimExample4.pop1.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample4_pop1_v_dat_f2.write('%e\t%e\t%e\t\n' % (py_v_time[i], py_v_pop1_0_testcell2_v_SimExample4_pop1_v_dat[i], py_v_pop1_1_testcell2_v_SimExample4_pop1_v_dat[i], ))
        f_SimExample4_pop1_v_dat_f2.close()
        print("Saved data to: SimExample4.pop1.v.dat")

        # ######################   File to save: SimExample4.pop0.v.dat (SimExample4_pop0_v_dat)
        py_v_pop0_0_testcell_v_SimExample4_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_0_testcell_v_SimExample4_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop0_1_testcell_v_SimExample4_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_1_testcell_v_SimExample4_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample4_pop0_v_dat_f2 = open('SimExample4.pop0.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample4_pop0_v_dat_f2.write('%e\t%e\t%e\t\n' % (py_v_time[i], py_v_pop0_0_testcell_v_SimExample4_pop0_v_dat[i], py_v_pop0_1_testcell_v_SimExample4_pop0_v_dat[i], ))
        f_SimExample4_pop0_v_dat_f2.close()
        print("Saved data to: SimExample4.pop0.v.dat")

        save_end = time.time()
        save_time = save_end - self.sim_end
        print("Finished saving results in %f seconds"%(save_time))

        self.report_file.write('StartTime=%s\n'%datetime.datetime.fromtimestamp(self.setup_start).strftime('%Y-%m-%d %H:%M:%S'))
        self.report_file.write('SetupTime=%s\n'%self.setup_time)
        self.report_file.write('RealSimulationTime=%s\n'%self.sim_time)
        self.report_file.write('SimulationSaveTime=%s\n'%save_time)
        self.report_file.close()

        print("Saving report of simulation to %s"%('report.SimExample4.txt'))

        print("Done")

        quit()


if __name__ == '__main__':

    ns = NeuronSimulation(tstop=1000.0, dt=0.01, seed=5678)

    ns.run()




from setuptools import setup

version = ""
for aline in open("neuromllite/__init__.py"):
    if "__version__ =" in aline:
        version = aline.split('"')[1]

setup(
    name="neuromllite",
    version=version,
    author="Padraig Gleeson",
    author_email="p.gleeson@gmail.com",
    packages=["neuromllite", "neuromllite.sweep", "neuromllite.gui"],
    entry_points={
        "console_scripts": ["nmllite-ui            = neuromllite.gui.NMLliteUI:main"]
    },
    url="https://github.com/NeuroML/NeuroMLlite",
    license="LICENSE.lesser",
    description="A common JSON/YAML based format for compact network specification, closely tied to NeuroML v2",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "libNeuroML>=0.2.54",
        "pyyaml",
        "numpy",
        "pyparsing<3",
        "modelspec>=0.2.2",
    ],
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering",
    ],
)



'''
Neuron simulator export for:

Components:
    null (Type: notes)
    synInput (Type: expTwoSynapse:  tauRise=0.001 (SI time) tauDecay=0.005 (SI time) peakTime=0.002011797390542625 (SI time) waveformFactor=1.8691859765265255 (dimensionless) gbase=8.0E-9 (SI conductance) erev=0.02 (SI voltage))
    iclamp0 (Type: pulseGenerator:  delay=0.02 (SI time) duration=0.8 (SI time) amplitude=1.0000000000000002E-10 (SI current))
    spikeGenPoisson (Type: spikeGeneratorPoisson:  averageRate=50.0 (SI per_time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    poissonFiringSyn (Type: poissonFiringSynapse:  averageRate=10.0 (SI per_time) averageIsi=0.1 (SI time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    poissonFiringSyn100Hz (Type: poissonFiringSynapse:  averageRate=100.0 (SI per_time) averageIsi=0.01 (SI time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    passiveChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    naChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    kChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    hhcell (Type: cell)
    ampa (Type: expTwoSynapse:  tauRise=0.001 (SI time) tauDecay=0.002 (SI time) peakTime=0.0013862943611198907 (SI time) waveformFactor=4.0 (dimensionless) gbase=2.0E-9 (SI conductance) erev=0.0 (SI voltage))
    Example3_Network (Type: network)
    SimExample3 (Type: Simulation:  length=1.0 (SI time) step=2.5E-5 (SI time))


    This NEURON file has been generated by org.neuroml.export (see https://github.com/NeuroML/org.neuroml.export)
         org.neuroml.export  v1.8.1
         org.neuroml.model   v1.8.1
         jLEMS               v0.10.6

'''

import neuron

import time
import datetime
import sys

import hashlib
h = neuron.h
h.load_file("stdlib.hoc")

h.load_file("stdgui.hoc")

h("objref p")
h("p = new PythonObject()")

class NeuronSimulation():

    def __init__(self, tstop, dt, seed=5678):

        print("\n    Starting simulation in NEURON of %sms generated from NeuroML2 model...\n"%tstop)

        self.setup_start = time.time()
        self.seed = seed
        import socket
        self.report_file = open('report.SimExample3.txt','w')
        print('Simulator version:  %s'%h.nrnversion())
        self.report_file.write('# Report of running simulation with %s\n'%h.nrnversion())
        self.report_file.write('Simulator=NEURON\n')
        self.report_file.write('SimulatorVersion=%s\n'%h.nrnversion())

        self.report_file.write('SimulationFile=%s\n'%__file__)
        self.report_file.write('PythonVersion=%s\n'%sys.version.replace('\n',' '))
        print('Python version:     %s'%sys.version.replace('\n',' '))
        self.report_file.write('NeuroMLExportVersion=1.8.1\n')
        self.report_file.write('SimulationSeed=%s\n'%self.seed)
        self.report_file.write('Hostname=%s\n'%socket.gethostname())
        self.randoms = []
        self.next_global_id = 0  # Used in Random123 classes for elements using random(), etc. 

        self.next_spiking_input_id = 0  # Used in Random123 classes for elements using random(), etc. 

        '''
        Adding simulation Component(id=SimExample3 type=Simulation) of network/component: Example3_Network (Type: network)
        
        '''
        # ######################   Population: pop0
        print("Population pop0 contains 5 instance(s) of component: hhcell of type: cell")

        h.load_file("hhcell.hoc")
        a_pop0 = []
        h("{ n_pop0 = 5 }")
        h("objectvar a_pop0[n_pop0]")
        for i in range(int(h.n_pop0)):
            h("a_pop0[%i] = new hhcell()"%i)
            h("access a_pop0[%i].soma"%i)

            self.next_global_id+=1

        h("{ a_pop0[0].position(966.45355, 44.07326, 7.49147) }")
        h("{ a_pop0[1].position(910.97595, 93.9269, 582.2276) }")
        h("{ a_pop0[2].position(671.5635, 8.393823, 766.48096) }")
        h("{ a_pop0[3].position(236.80977, 3.081402, 788.7727) }")
        h("{ a_pop0[4].position(346.08896, 62.328148, 615.8157) }")

        h("proc initialiseV_pop0() { for i = 0, n_pop0-1 { a_pop0[i].set_initial_v() } }")
        h("objref fih_pop0")
        h('{fih_pop0 = new FInitializeHandler(0, "initialiseV_pop0()")}')

        h("proc initialiseIons_pop0() { for i = 0, n_pop0-1 { a_pop0[i].set_initial_ion_properties() } }")
        h("objref fih_ion_pop0")
        h('{fih_ion_pop0 = new FInitializeHandler(1, "initialiseIons_pop0()")}')

        # ######################   Population: pop1
        print("Population pop1 contains 10 instance(s) of component: hhcell of type: cell")

        h.load_file("hhcell.hoc")
        a_pop1 = []
        h("{ n_pop1 = 10 }")
        h("objectvar a_pop1[n_pop1]")
        for i in range(int(h.n_pop1)):
            h("a_pop1[%i] = new hhcell()"%i)
            h("access a_pop1[%i].soma"%i)

            self.next_global_id+=1

        h("{ a_pop1[0].position(148.55464, 218.30907, 114.41297) }")
        h("{ a_pop1[1].position(14.61878, 248.67516, 964.90155) }")
        h("{ a_pop1[2].position(64.56228, 254.10883, 465.89856) }")
        h("{ a_pop1[3].position(601.46344, 208.89288, 579.0027) }")
        h("{ a_pop1[4].position(269.5855, 255.64325, 644.6342) }")
        h("{ a_pop1[5].position(481.03638, 235.52391, 249.15211) }")
        h("{ a_pop1[6].position(933.5155, 245.3388, 530.1612) }")
        h("{ a_pop1[7].position(19.299566, 250.8102, 5.780237) }")
        h("{ a_pop1[8].position(143.76843, 247.2827, 377.34744) }")
        h("{ a_pop1[9].position(54.175198, 258.75284, 164.00322) }")

        h("proc initialiseV_pop1() { for i = 0, n_pop1-1 { a_pop1[i].set_initial_v() } }")
        h("objref fih_pop1")
        h('{fih_pop1 = new FInitializeHandler(0, "initialiseV_pop1()")}')

        h("proc initialiseIons_pop1() { for i = 0, n_pop1-1 { a_pop1[i].set_initial_ion_properties() } }")
        h("objref fih_ion_pop1")
        h('{fih_ion_pop1 = new FInitializeHandler(1, "initialiseIons_pop1()")}')

        # ######################   Projection: proj0
        print("Adding projection: proj0, from pop0 to pop1 with synapse ampa, 26 connection(s)")

        h("objectvar syn_proj0_ampa[26]")

        h("objectvar netConn_proj0_ampa[26]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on a_pop0[0].soma] -> cell 1, seg 0 (0.5) [0.5 on a_pop1[1].soma], weight: 1.0, delay 0.0
        h("a_pop1[1].soma syn_proj0_ampa[0] = new ampa(0.5)")
        h("a_pop0[0].soma a_pop1[1].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[0], -20.0, 0.0, 1.0))") # *->cell

        # Connection 1: cell 0, seg 0 (0.5) [0.5 on a_pop0[0].soma] -> cell 5, seg 0 (0.5) [0.5 on a_pop1[5].soma], weight: 1.0, delay 0.0
        h("a_pop1[5].soma syn_proj0_ampa[1] = new ampa(0.5)")
        h("a_pop0[0].soma a_pop1[5].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[1], -20.0, 0.0, 1.0))") # *->cell

        # Connection 2: cell 0, seg 0 (0.5) [0.5 on a_pop0[0].soma] -> cell 6, seg 0 (0.5) [0.5 on a_pop1[6].soma], weight: 1.0, delay 0.0
        h("a_pop1[6].soma syn_proj0_ampa[2] = new ampa(0.5)")
        h("a_pop0[0].soma a_pop1[6].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[2], -20.0, 0.0, 1.0))") # *->cell

        # Connection 3: cell 0, seg 0 (0.5) [0.5 on a_pop0[0].soma] -> cell 7, seg 0 (0.5) [0.5 on a_pop1[7].soma], weight: 1.0, delay 0.0
        h("a_pop1[7].soma syn_proj0_ampa[3] = new ampa(0.5)")
        h("a_pop0[0].soma a_pop1[7].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[3], -20.0, 0.0, 1.0))") # *->cell

        # Connection 4: cell 0, seg 0 (0.5) [0.5 on a_pop0[0].soma] -> cell 8, seg 0 (0.5) [0.5 on a_pop1[8].soma], weight: 1.0, delay 0.0
        h("a_pop1[8].soma syn_proj0_ampa[4] = new ampa(0.5)")
        h("a_pop0[0].soma a_pop1[8].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[4], -20.0, 0.0, 1.0))") # *->cell

        # Connection 5: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 0, seg 0 (0.5) [0.5 on a_pop1[0].soma], weight: 1.0, delay 0.0
        h("a_pop1[0].soma syn_proj0_ampa[5] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[0].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[5], -20.0, 0.0, 1.0))") # *->cell

        # Connection 6: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 1, seg 0 (0.5) [0.5 on a_pop1[1].soma], weight: 1.0, delay 0.0
        h("a_pop1[1].soma syn_proj0_ampa[6] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[1].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[6], -20.0, 0.0, 1.0))") # *->cell

        # Connection 7: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 3, seg 0 (0.5) [0.5 on a_pop1[3].soma], weight: 1.0, delay 0.0
        h("a_pop1[3].soma syn_proj0_ampa[7] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[3].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[7], -20.0, 0.0, 1.0))") # *->cell

        # Connection 8: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 5, seg 0 (0.5) [0.5 on a_pop1[5].soma], weight: 1.0, delay 0.0
        h("a_pop1[5].soma syn_proj0_ampa[8] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[5].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[8], -20.0, 0.0, 1.0))") # *->cell

        # Connection 9: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 6, seg 0 (0.5) [0.5 on a_pop1[6].soma], weight: 1.0, delay 0.0
        h("a_pop1[6].soma syn_proj0_ampa[9] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[6].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[9], -20.0, 0.0, 1.0))") # *->cell

        # Connection 10: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 7, seg 0 (0.5) [0.5 on a_pop1[7].soma], weight: 1.0, delay 0.0
        h("a_pop1[7].soma syn_proj0_ampa[10] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[7].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[10], -20.0, 0.0, 1.0))") # *->cell

        # Connection 11: cell 1, seg 0 (0.5) [0.5 on a_pop0[1].soma] -> cell 8, seg 0 (0.5) [0.5 on a_pop1[8].soma], weight: 1.0, delay 0.0
        h("a_pop1[8].soma syn_proj0_ampa[11] = new ampa(0.5)")
        h("a_pop0[1].soma a_pop1[8].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[11], -20.0, 0.0, 1.0))") # *->cell

        # Connection 12: cell 2, seg 0 (0.5) [0.5 on a_pop0[2].soma] -> cell 0, seg 0 (0.5) [0.5 on a_pop1[0].soma], weight: 1.0, delay 0.0
        h("a_pop1[0].soma syn_proj0_ampa[12] = new ampa(0.5)")
        h("a_pop0[2].soma a_pop1[0].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[12], -20.0, 0.0, 1.0))") # *->cell

        # Connection 13: cell 2, seg 0 (0.5) [0.5 on a_pop0[2].soma] -> cell 2, seg 0 (0.5) [0.5 on a_pop1[2].soma], weight: 1.0, delay 0.0
        h("a_pop1[2].soma syn_proj0_ampa[13] = new ampa(0.5)")
        h("a_pop0[2].soma a_pop1[2].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[13], -20.0, 0.0, 1.0))") # *->cell

        # Connection 14: cell 2, seg 0 (0.5) [0.5 on a_pop0[2].soma] -> cell 5, seg 0 (0.5) [0.5 on a_pop1[5].soma], weight: 1.0, delay 0.0
        h("a_pop1[5].soma syn_proj0_ampa[14] = new ampa(0.5)")
        h("a_pop0[2].soma a_pop1[5].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[14], -20.0, 0.0, 1.0))") # *->cell

        # Connection 15: cell 2, seg 0 (0.5) [0.5 on a_pop0[2].soma] -> cell 8, seg 0 (0.5) [0.5 on a_pop1[8].soma], weight: 1.0, delay 0.0
        h("a_pop1[8].soma syn_proj0_ampa[15] = new ampa(0.5)")
        h("a_pop0[2].soma a_pop1[8].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[15], -20.0, 0.0, 1.0))") # *->cell

        # Connection 16: cell 2, seg 0 (0.5) [0.5 on a_pop0[2].soma] -> cell 9, seg 0 (0.5) [0.5 on a_pop1[9].soma], weight: 1.0, delay 0.0
        h("a_pop1[9].soma syn_proj0_ampa[16] = new ampa(0.5)")
        h("a_pop0[2].soma a_pop1[9].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[16], -20.0, 0.0, 1.0))") # *->cell

        # Connection 17: cell 3, seg 0 (0.5) [0.5 on a_pop0[3].soma] -> cell 0, seg 0 (0.5) [0.5 on a_pop1[0].soma], weight: 1.0, delay 0.0
        h("a_pop1[0].soma syn_proj0_ampa[17] = new ampa(0.5)")
        h("a_pop0[3].soma a_pop1[0].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[17], -20.0, 0.0, 1.0))") # *->cell

        # Connection 18: cell 3, seg 0 (0.5) [0.5 on a_pop0[3].soma] -> cell 1, seg 0 (0.5) [0.5 on a_pop1[1].soma], weight: 1.0, delay 0.0
        h("a_pop1[1].soma syn_proj0_ampa[18] = new ampa(0.5)")
        h("a_pop0[3].soma a_pop1[1].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[18], -20.0, 0.0, 1.0))") # *->cell

        # Connection 19: cell 3, seg 0 (0.5) [0.5 on a_pop0[3].soma] -> cell 2, seg 0 (0.5) [0.5 on a_pop1[2].soma], weight: 1.0, delay 0.0
        h("a_pop1[2].soma syn_proj0_ampa[19] = new ampa(0.5)")
        h("a_pop0[3].soma a_pop1[2].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[19], -20.0, 0.0, 1.0))") # *->cell

        # Connection 20: cell 3, seg 0 (0.5) [0.5 on a_pop0[3].soma] -> cell 5, seg 0 (0.5) [0.5 on a_pop1[5].soma], weight: 1.0, delay 0.0
        h("a_pop1[5].soma syn_proj0_ampa[20] = new ampa(0.5)")
        h("a_pop0[3].soma a_pop1[5].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[20], -20.0, 0.0, 1.0))") # *->cell

        # Connection 21: cell 3, seg 0 (0.5) [0.5 on a_pop0[3].soma] -> cell 9, seg 0 (0.5) [0.5 on a_pop1[9].soma], weight: 1.0, delay 0.0
        h("a_pop1[9].soma syn_proj0_ampa[21] = new ampa(0.5)")
        h("a_pop0[3].soma a_pop1[9].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[21], -20.0, 0.0, 1.0))") # *->cell

        # Connection 22: cell 4, seg 0 (0.5) [0.5 on a_pop0[4].soma] -> cell 2, seg 0 (0.5) [0.5 on a_pop1[2].soma], weight: 1.0, delay 0.0
        h("a_pop1[2].soma syn_proj0_ampa[22] = new ampa(0.5)")
        h("a_pop0[4].soma a_pop1[2].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[22], -20.0, 0.0, 1.0))") # *->cell

        # Connection 23: cell 4, seg 0 (0.5) [0.5 on a_pop0[4].soma] -> cell 4, seg 0 (0.5) [0.5 on a_pop1[4].soma], weight: 1.0, delay 0.0
        h("a_pop1[4].soma syn_proj0_ampa[23] = new ampa(0.5)")
        h("a_pop0[4].soma a_pop1[4].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[23], -20.0, 0.0, 1.0))") # *->cell

        # Connection 24: cell 4, seg 0 (0.5) [0.5 on a_pop0[4].soma] -> cell 5, seg 0 (0.5) [0.5 on a_pop1[5].soma], weight: 1.0, delay 0.0
        h("a_pop1[5].soma syn_proj0_ampa[24] = new ampa(0.5)")
        h("a_pop0[4].soma a_pop1[5].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[24], -20.0, 0.0, 1.0))") # *->cell

        # Connection 25: cell 4, seg 0 (0.5) [0.5 on a_pop0[4].soma] -> cell 9, seg 0 (0.5) [0.5 on a_pop1[9].soma], weight: 1.0, delay 0.0
        h("a_pop1[9].soma syn_proj0_ampa[25] = new ampa(0.5)")
        h("a_pop0[4].soma a_pop1[9].synlist.append(new NetCon(&v(0.5), syn_proj0_ampa[25], -20.0, 0.0, 1.0))") # *->cell

        print("Processing 1 input lists")

        # ######################   Input List: stim_pop0
        # Adding single input: Component(id=0 type=input)
        h("objref stim_pop0_0")
        h("a_pop0[0].soma { stim_pop0_0 = new poissonFiringSyn(0.5) } ")
        rand = h.Random()
        self.randoms.append(rand)
        #print("Seeding random generator on stim_pop0_0 with stim seed %s"%(self.seed))
        self._init_stim_randomizer(rand,"stim_pop0",0, self.seed)
        rand.negexp(1)
        h.stim_pop0_0.noiseFromRandom(rand)
        self.next_spiking_input_id+=1

        # Adding single input: Component(id=1 type=input)
        h("objref stim_pop0_1")
        h("a_pop0[1].soma { stim_pop0_1 = new poissonFiringSyn(0.5) } ")
        rand = h.Random()
        self.randoms.append(rand)
        #print("Seeding random generator on stim_pop0_1 with stim seed %s"%(self.seed))
        self._init_stim_randomizer(rand,"stim_pop0",1, self.seed)
        rand.negexp(1)
        h.stim_pop0_1.noiseFromRandom(rand)
        self.next_spiking_input_id+=1

        # Adding single input: Component(id=2 type=input)
        h("objref stim_pop0_2")
        h("a_pop0[2].soma { stim_pop0_2 = new poissonFiringSyn(0.5) } ")
        rand = h.Random()
        self.randoms.append(rand)
        #print("Seeding random generator on stim_pop0_2 with stim seed %s"%(self.seed))
        self._init_stim_randomizer(rand,"stim_pop0",2, self.seed)
        rand.negexp(1)
        h.stim_pop0_2.noiseFromRandom(rand)
        self.next_spiking_input_id+=1

        # Adding single input: Component(id=3 type=input)
        h("objref stim_pop0_3")
        h("a_pop0[3].soma { stim_pop0_3 = new poissonFiringSyn(0.5) } ")
        rand = h.Random()
        self.randoms.append(rand)
        #print("Seeding random generator on stim_pop0_3 with stim seed %s"%(self.seed))
        self._init_stim_randomizer(rand,"stim_pop0",3, self.seed)
        rand.negexp(1)
        h.stim_pop0_3.noiseFromRandom(rand)
        self.next_spiking_input_id+=1

        # Adding single input: Component(id=4 type=input)
        h("objref stim_pop0_4")
        h("a_pop0[4].soma { stim_pop0_4 = new poissonFiringSyn(0.5) } ")
        rand = h.Random()
        self.randoms.append(rand)
        #print("Seeding random generator on stim_pop0_4 with stim seed %s"%(self.seed))
        self._init_stim_randomizer(rand,"stim_pop0",4, self.seed)
        rand.negexp(1)
        h.stim_pop0_4.noiseFromRandom(rand)
        self.next_spiking_input_id+=1

        print("Finished processing 1 input lists")

        trec = h.Vector()
        trec.record(h._ref_t)

        h.tstop = tstop

        h.dt = dt

        h.steps_per_ms = 1/h.dt



        # ######################   File to save: SimExample3.pop0.v.dat (SimExample3_pop0_v_dat)
        # Column: pop0/0/hhcell/v
        h(' objectvar v_pop0_0_hhcell_v_SimExample3_pop0_v_dat ')
        h(' { v_pop0_0_hhcell_v_SimExample3_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_0_hhcell_v_SimExample3_pop0_v_dat.record(&a_pop0[0].soma.v(0.5)) } ')
        h.v_pop0_0_hhcell_v_SimExample3_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop0/1/hhcell/v
        h(' objectvar v_pop0_1_hhcell_v_SimExample3_pop0_v_dat ')
        h(' { v_pop0_1_hhcell_v_SimExample3_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_1_hhcell_v_SimExample3_pop0_v_dat.record(&a_pop0[1].soma.v(0.5)) } ')
        h.v_pop0_1_hhcell_v_SimExample3_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop0/2/hhcell/v
        h(' objectvar v_pop0_2_hhcell_v_SimExample3_pop0_v_dat ')
        h(' { v_pop0_2_hhcell_v_SimExample3_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_2_hhcell_v_SimExample3_pop0_v_dat.record(&a_pop0[2].soma.v(0.5)) } ')
        h.v_pop0_2_hhcell_v_SimExample3_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop0/3/hhcell/v
        h(' objectvar v_pop0_3_hhcell_v_SimExample3_pop0_v_dat ')
        h(' { v_pop0_3_hhcell_v_SimExample3_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_3_hhcell_v_SimExample3_pop0_v_dat.record(&a_pop0[3].soma.v(0.5)) } ')
        h.v_pop0_3_hhcell_v_SimExample3_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop0/4/hhcell/v
        h(' objectvar v_pop0_4_hhcell_v_SimExample3_pop0_v_dat ')
        h(' { v_pop0_4_hhcell_v_SimExample3_pop0_v_dat = new Vector() } ')
        h(' { v_pop0_4_hhcell_v_SimExample3_pop0_v_dat.record(&a_pop0[4].soma.v(0.5)) } ')
        h.v_pop0_4_hhcell_v_SimExample3_pop0_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: time.dat (time)
        # Column: time
        h(' objectvar v_time ')
        h(' { v_time = new Vector() } ')
        h(' { v_time.record(&t) } ')
        h.v_time.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample3.pop1.v.dat (SimExample3_pop1_v_dat)
        # Column: pop1/0/hhcell/v
        h(' objectvar v_pop1_0_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_0_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_0_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[0].soma.v(0.5)) } ')
        h.v_pop1_0_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/1/hhcell/v
        h(' objectvar v_pop1_1_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_1_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_1_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[1].soma.v(0.5)) } ')
        h.v_pop1_1_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/2/hhcell/v
        h(' objectvar v_pop1_2_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_2_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_2_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[2].soma.v(0.5)) } ')
        h.v_pop1_2_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/3/hhcell/v
        h(' objectvar v_pop1_3_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_3_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_3_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[3].soma.v(0.5)) } ')
        h.v_pop1_3_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/4/hhcell/v
        h(' objectvar v_pop1_4_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_4_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_4_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[4].soma.v(0.5)) } ')
        h.v_pop1_4_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/5/hhcell/v
        h(' objectvar v_pop1_5_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_5_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_5_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[5].soma.v(0.5)) } ')
        h.v_pop1_5_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/6/hhcell/v
        h(' objectvar v_pop1_6_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_6_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_6_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[6].soma.v(0.5)) } ')
        h.v_pop1_6_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/7/hhcell/v
        h(' objectvar v_pop1_7_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_7_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_7_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[7].soma.v(0.5)) } ')
        h.v_pop1_7_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/8/hhcell/v
        h(' objectvar v_pop1_8_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_8_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_8_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[8].soma.v(0.5)) } ')
        h.v_pop1_8_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: pop1/9/hhcell/v
        h(' objectvar v_pop1_9_hhcell_v_SimExample3_pop1_v_dat ')
        h(' { v_pop1_9_hhcell_v_SimExample3_pop1_v_dat = new Vector() } ')
        h(' { v_pop1_9_hhcell_v_SimExample3_pop1_v_dat.record(&a_pop1[9].soma.v(0.5)) } ')
        h.v_pop1_9_hhcell_v_SimExample3_pop1_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        self.initialized = False

        self.sim_end = -1 # will be overwritten

        setup_end = time.time()
        self.setup_time = setup_end - self.setup_start
        print("Setting up the network to simulate took %f seconds"%(self.setup_time))

    def run(self):

        self.initialized = True
        sim_start = time.time()
        print("Running a simulation of %sms (dt = %sms; seed=%s)" % (h.tstop, h.dt, self.seed))

        try:
            h.run()
        except Exception as e:
            print("Exception running NEURON: %s" % (e))
            quit()


        self.sim_end = time.time()
        self.sim_time = self.sim_end - sim_start
        print("Finished NEURON simulation in %f seconds (%f mins)..."%(self.sim_time, self.sim_time/60.0))

        try:
            self.save_results()
        except Exception as e:
            print("Exception saving results of NEURON simulation: %s" % (e))
            quit()


    def advance(self):

        if not self.initialized:
            h.finitialize()
            self.initialized = True

        h.fadvance()


    ###############################################################################
    # Hash function to use in generation of random value
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _id32 (self,obj): 
        return int(hashlib.md5(obj.encode('utf-8')).hexdigest()[0:8],16)  # convert 8 first chars of md5 hash in base 16 to int


    ###############################################################################
    # Initialize the stim randomizer
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _init_stim_randomizer(self,rand, stimType, gid, seed): 
        #print("INIT STIM  %s; %s; %s; %s"%(rand, stimType, gid, seed))
        rand.Random123(self._id32(stimType), gid, seed)


    def save_results(self):

        print("Saving results at t=%s..."%h.t)

        if self.sim_end < 0: self.sim_end = time.time()


        # ######################   File to save: time.dat (time)
        py_v_time = [ t/1000 for t in h.v_time.to_python() ]  # Convert to Python list for speed...

        f_time_f2 = open('time.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_time_f2.write('%f'% py_v_time[i])  # Save in SI units...
        f_time_f2.close()
        print("Saved data to: time.dat")

        # ######################   File to save: SimExample3.pop0.v.dat (SimExample3_pop0_v_dat)
        py_v_pop0_0_hhcell_v_SimExample3_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_0_hhcell_v_SimExample3_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop0_1_hhcell_v_SimExample3_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_1_hhcell_v_SimExample3_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop0_2_hhcell_v_SimExample3_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_2_hhcell_v_SimExample3_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop0_3_hhcell_v_SimExample3_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_3_hhcell_v_SimExample3_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop0_4_hhcell_v_SimExample3_pop0_v_dat = [ float(x  / 1000.0) for x in h.v_pop0_4_hhcell_v_SimExample3_pop0_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample3_pop0_v_dat_f2 = open('SimExample3.pop0.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample3_pop0_v_dat_f2.write('%e\t%e\t%e\t%e\t%e\t%e\t\n' % (py_v_time[i], py_v_pop0_0_hhcell_v_SimExample3_pop0_v_dat[i], py_v_pop0_1_hhcell_v_SimExample3_pop0_v_dat[i], py_v_pop0_2_hhcell_v_SimExample3_pop0_v_dat[i], py_v_pop0_3_hhcell_v_SimExample3_pop0_v_dat[i], py_v_pop0_4_hhcell_v_SimExample3_pop0_v_dat[i], ))
        f_SimExample3_pop0_v_dat_f2.close()
        print("Saved data to: SimExample3.pop0.v.dat")

        # ######################   File to save: SimExample3.pop1.v.dat (SimExample3_pop1_v_dat)
        py_v_pop1_0_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_0_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_1_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_1_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_2_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_2_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_3_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_3_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_4_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_4_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_5_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_5_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_6_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_6_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_7_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_7_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_8_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_8_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_pop1_9_hhcell_v_SimExample3_pop1_v_dat = [ float(x  / 1000.0) for x in h.v_pop1_9_hhcell_v_SimExample3_pop1_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample3_pop1_v_dat_f2 = open('SimExample3.pop1.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample3_pop1_v_dat_f2.write('%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t\n' % (py_v_time[i], py_v_pop1_0_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_1_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_2_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_3_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_4_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_5_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_6_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_7_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_8_hhcell_v_SimExample3_pop1_v_dat[i], py_v_pop1_9_hhcell_v_SimExample3_pop1_v_dat[i], ))
        f_SimExample3_pop1_v_dat_f2.close()
        print("Saved data to: SimExample3.pop1.v.dat")

        save_end = time.time()
        save_time = save_end - self.sim_end
        print("Finished saving results in %f seconds"%(save_time))

        self.report_file.write('StartTime=%s\n'%datetime.datetime.fromtimestamp(self.setup_start).strftime('%Y-%m-%d %H:%M:%S'))
        self.report_file.write('SetupTime=%s\n'%self.setup_time)
        self.report_file.write('RealSimulationTime=%s\n'%self.sim_time)
        self.report_file.write('SimulationSaveTime=%s\n'%save_time)
        self.report_file.close()

        print("Saving report of simulation to %s"%('report.SimExample3.txt'))

        print("Done")

        quit()


if __name__ == '__main__':

    ns = NeuronSimulation(tstop=1000.0, dt=0.025, seed=5678)

    ns.run()




from os.path import dirname
from os.path import realpath
import sys

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

from matplotlib.backends.backend_qt5agg import FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar

import matplotlib
from matplotlib.figure import Figure

from neuromllite.utils import load_network_json
from neuromllite.utils import load_simulation_json
from neuromllite.utils import evaluate
from neuromllite.utils import print_v, print_
from neuromllite.utils import is_spiking_input_population
from pyneuroml.pynml import get_next_hex_color

from functools import partial


class ParameterSpinBox(QDoubleSpinBox):

    # TODO: handle a spinner on values like -60mV etc.
    def textFromValue(self, value):
        return "%s" % value


class NMLliteUI(QWidget):

    default_vals = {}

    simulators = [
        "jNeuroML",
        "jNeuroML_NEURON",
        "jNeuroML_NetPyNE",
        "NetPyNE",
        "PyNN_NEURON",
        "PyNN_NEST",
        "PyNN_Brian",
        "Arbor",
        "BindsNET",
    ]
    """    'jNeuroML_Brian2',"""

    LEMS_VIEW_TAB = "Lems View"
    IMAGE_3D_TAB = "3D image"
    GRAPH_TAB = "Graph"

    verbose = False

    def updated_param(self, p):
        """A parameter has been updated"""

        print_("=====   Param %s changed" % p, self.verbose)

        if self.autoRunCheckBox.isChecked():
            self.runSimulation()
        else:
            print_("Nothing changing...", self.verbose)

        self.update_net_sim()
        if self.tabs.currentWidget() == self.nmlliteTab:
            self.update_network_json()
            self.update_simulation_json()

    def update_network_json(self):

        self.nmlliteNetText.clear()
        try:
            j = self.network.to_json()
            self.nmlliteNetText.insertPlainText(j)
        except Exception as e:
            self.nmlliteNetText.insertPlainText("Error parsing model: %s" % e)

    def update_simulation_json(self):

        self.nmlliteSimText.clear()
        try:
            j = self.simulation.to_json()
            self.nmlliteSimText.insertPlainText(j)
        except Exception as e:
            self.nmlliteSimText.insertPlainText("Error parsing model: %s" % e)

    all_tabs = {}
    # all_tab_layouts = {}
    all_figures = {}
    all_figures_2dplots = {}
    all_canvases = {}
    all_layouts = {}
    all_options_layouts = {}
    all_image_qlabels = {}
    all_image_scales = {}

    current_traces_shown = {}
    current_traces_colours = {}

    def add_tab(
        self,
        name,
        parent_tab_holder,
        image=False,
        figure=False,
        toolbar=False,
        options=False,
    ):

        if name in self.all_tabs:
            raise Exception("The name for a tab: %s is already taken!" % name)

        thisTab = QWidget()
        parent_tab_holder.addTab(thisTab, name)
        self.all_tabs[name] = thisTab

        topLayout = QGridLayout()
        thisTab.setLayout(topLayout)
        self.all_layouts[name] = topLayout

        if options:
            thisOptionsLayout = QGridLayout()
            topLayout.addLayout(thisOptionsLayout, 0, 0)
            thisOptionsLayout.addWidget(QLabel("Options:"), 0, 0)
            self.all_options_layouts[name] = thisOptionsLayout

        if figure:
            thisFigure = Figure()
            thisCanvas = FigureCanvas(thisFigure)
            if toolbar:
                thisToolbar = NavigationToolbar(thisCanvas, self)

            self.all_canvases[name] = thisCanvas
            self.all_figures[name] = thisFigure
            if options:
                thisFigureLayout = QGridLayout()
                topLayout.addLayout(thisFigureLayout, 1, 0)
                thisFigureLayout.addWidget(thisCanvas)
                if toolbar:
                    thisFigureLayout.addWidget(thisToolbar)
            else:
                topLayout.addWidget(thisCanvas)
                if toolbar:
                    topLayout.addWidget(thisToolbar)

        if image:
            label = QLabel(
                "An image will be generated here. Push the appropriate button on the left"
            )
            label.setBackgroundRole(QPalette.Base)
            label.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
            label.setScaledContents(True)
            self.all_image_qlabels[name] = label

            scrollArea = QScrollArea()
            scrollArea.setBackgroundRole(QPalette.Light)
            scrollArea.setWidget(label)
            scrollArea.setVisible(True)

            scroolLayout = QGridLayout()
            topLayout.addLayout(scroolLayout, 1, 0)
            scroolLayout.addWidget(scrollArea, 0, 0)

            if options:
                plus_button = QPushButton("+")
                plus_button.show()
                plus_button.clicked.connect(partial(self.scale_image, name, "+"))
                thisOptionsLayout.addWidget(plus_button, 0, 1)
                orig_button = QPushButton("1")
                orig_button.show()
                orig_button.clicked.connect(partial(self.scale_image, name, "1"))
                thisOptionsLayout.addWidget(orig_button, 0, 2)
                min_button = QPushButton("-")
                min_button.show()
                min_button.clicked.connect(partial(self.scale_image, name, "-"))
                thisOptionsLayout.addWidget(min_button, 0, 3)

        if options:
            return thisOptionsLayout

    def scale_image(self, tab_reference, action):
        scaleFactor = self.all_image_scales[tab_reference]

        print("Scaling with: %s on %s, was %s" % (action, tab_reference, scaleFactor))
        if action == "+":
            self.all_image_scales[tab_reference] *= 1.25
        if action == "-":
            self.all_image_scales[tab_reference] *= 0.8
        if action == "1":
            self.all_image_scales[tab_reference] = 1

        label = self.all_image_qlabels[tab_reference]
        label.resize(self.all_image_scales[tab_reference] * label.pixmap().size())

    def add_image(self, image_file, tab_reference):
        # Inspired by https://gist.github.com/acbetter/32c575803ec361c3e82064e60db4e3e0

        print_v("Displaying an image: %s" % (image_file))

        image = QImage(image_file)
        pixmap = QPixmap.fromImage(image)

        label = self.all_image_qlabels[tab_reference]
        # pixmap = pixmap.scaledToWidth(min(pixmap.width(), 800), Qt.SmoothTransformation)
        label.setPixmap(pixmap)
        self.all_image_scales[tab_reference] = 1
        scaleFactor = self.all_image_scales[tab_reference]

        label.resize(scaleFactor * label.pixmap().size())

    def get_value_entry(self, name, value, entry_map):
        """Create a graphical element for displaying/setting values"""

        simple = False
        # simple = True

        if simple:
            entry = QLineEdit()
            entry_map[name] = entry
            entry.setText(str(value))
            entry.textChanged.connect(self.updated_param)

        else:

            try:
                entry = ParameterSpinBox()
                entry_map[name] = entry
                entry.setDecimals(18)
                entry.setMaximum(1e16)
                entry.setMinimum(-1e16)
                entry.setSingleStep(value / 20.0)
                entry.setValue(float(value))
                entry.valueChanged.connect(self.updated_param)

            except Exception as e:
                print_v("Error: %s" % e)

                entry = QLineEdit()
                entry_map[name] = entry
                entry.setText(str(value))
                entry.textChanged.connect(self.updated_param)

        entry.setToolTip("Parameter: %s (initial value: %s)" % (name, value))

        print_(
            "Created value entry widget for %s (= %s): %s (%s)"
            % (name, value, entry, entry.text()),
            self.verbose,
        )
        return entry

    def dialog_popup(self, message):
        dialog = QMessageBox()
        dialog.setIcon(QMessageBox.Warning)
        dialog.setWindowTitle("Message")
        dialog.setText(message)
        dialog.exec_()

    def __init__(self, nml_sim_file, parent=None):
        """Constructor for the GUI"""

        super(NMLliteUI, self).__init__(parent)

        self.SPIKES_RASTERPLOT = "Rasterplot"
        self.SPIKES_POP_RATE_AVE = "Pop rate averages"

        # print('Styles available: %s'%QStyleFactory.keys())
        QApplication.setStyle(QStyleFactory.create("Fusion"))

        header_font = QFont()
        header_font.setBold(True)

        self.backup_colors = {}  # to ensure consistent random colors for traces...

        self.simulation = load_simulation_json(nml_sim_file)
        self.sim_base_dir = dirname(nml_sim_file)
        if len(self.sim_base_dir) == 0:
            self.sim_base_dir = "."

        self.network = load_network_json(
            "%s/%s" % (self.sim_base_dir, self.simulation.network)
        )

        nameLabel = QLabel("NMLlite file: %s" % realpath(nml_sim_file))
        nameLabel.setFont(header_font)

        paramLayout = QGridLayout()
        topLayout = QGridLayout()
        midLayout = QGridLayout()

        self.tabs = QTabWidget()
        self.nmlliteTab = QWidget()
        self.nml2Tab = QWidget()
        self.matrixTab = QWidget()

        self.tabs.resize(300, 200)

        # Add tabs
        self.simTab = QWidget()
        self.tabs.addTab(self.simTab, "Simulation")

        self.simTabLayout = QGridLayout()
        self.simTab.setLayout(self.simTabLayout)

        self.plotTabs = QTabWidget()

        self.tracesTab = QWidget()
        self.plotTabs.addTab(self.tracesTab, "Traces")
        self.heatmapTab = QWidget()
        self.plotTabs.addTab(self.heatmapTab, "Heatmap")

        if self.simulation.plots2D is not None:

            self.plot2DTab = QTabWidget()
            self.plotTabs.addTab(self.plot2DTab, "2D plots")

            self.plot2DTabLayout = QGridLayout()
            self.plot2DTab.setLayout(self.plot2DTabLayout)

            for plot2D in self.simulation.plots2D:
                info = self.simulation.plots2D[plot2D]
                pLayout = self.add_tab(
                    plot2D, self.plot2DTab, figure=True, toolbar=True, options=True
                )

        if self.simulation.plots3D is not None:

            self.plot3DTab = QTabWidget()
            self.plotTabs.addTab(self.plot3DTab, "3D plots")

            self.plot3DTabLayout = QGridLayout()
            self.plot3DTab.setLayout(self.plot3DTabLayout)

            for plot3D in self.simulation.plots3D:
                info = self.simulation.plots3D[plot3D]
                pLayout = self.add_tab(
                    plot3D, self.plot3DTab, figure=True, toolbar=False, options=True
                )

        offset_opt = 1
        rasterOptionsLayout = self.add_tab(
            self.SPIKES_RASTERPLOT,
            self.plotTabs,
            figure=True,
            toolbar=True,
            options=True,
        )
        self.rasterLegend = QCheckBox("Show legend")
        self.rasterLegend.setChecked(True)
        rasterOptionsLayout.addWidget(self.rasterLegend, 0, 0 + offset_opt)
        self.rasterLegend.toggled.connect(self.replotSimResults)
        self.rasterInPops = QCheckBox("Include input pops")
        self.rasterInPops.setChecked(True)
        rasterOptionsLayout.addWidget(self.rasterInPops, 0, 1 + offset_opt)
        self.rasterInPops.toggled.connect(self.replotSimResults)

        spikeStatOptionsLayout = self.add_tab(
            self.SPIKES_POP_RATE_AVE,
            self.plotTabs,
            figure=True,
            toolbar=True,
            options=True,
        )
        self.spikeStatInPops = QCheckBox("Include input pops")
        self.spikeStatInPops.setChecked(True)
        spikeStatOptionsLayout.addWidget(self.spikeStatInPops, 0, 0 + offset_opt)
        self.spikeStatInPops.toggled.connect(self.replotSimResults)

        self.simTabLayout.addWidget(self.plotTabs)

        self.tracesTabTopLayout = QGridLayout()
        self.tracesTab.setLayout(self.tracesTabTopLayout)

        self.tracesTabOptionsLayout = QGridLayout()
        self.tracesTabTopLayout.addLayout(self.tracesTabOptionsLayout, 0, 0)

        self.showTracesLegend = QCheckBox("Legend")
        self.tracesTabOptionsLayout.addWidget(self.showTracesLegend, 0, 0)

        self.traceSelectButton = QPushButton("Select traces...")
        self.traceSelectButton.show()
        self.traceSelectButton.setEnabled(False)

        self.traceSelectButton.clicked.connect(self.traceSelect)
        self.tracesTabOptionsLayout.addWidget(self.traceSelectButton, 0, 1)

        self.tracesTabLayout = QGridLayout()
        self.tracesTabTopLayout.addLayout(self.tracesTabLayout, 1, 0)

        self.heatmapLayout = QGridLayout()
        self.heatmapTab.setLayout(self.heatmapLayout)
        self.heatmapColorbar = None

        self.tracesFigure = Figure()
        self.tracesCanvas = FigureCanvas(self.tracesFigure)
        self.tracesToolbar = NavigationToolbar(self.tracesCanvas, self)
        self.tracesTabLayout.addWidget(self.tracesCanvas)
        self.tracesTabLayout.addWidget(self.tracesToolbar)

        self.heatmapFigure = Figure()
        self.heatmapCanvas = FigureCanvas(self.heatmapFigure)
        self.heatmapLayout.addWidget(self.heatmapCanvas)

        self.add_tab(self.GRAPH_TAB, self.tabs, image=True, options=True)

        graphTabOptionsLayout = self.all_options_layouts[self.GRAPH_TAB]

        offset_opt = 4
        graphTabOptionsLayout.addWidget(QLabel("Graph level:"), 0, offset_opt + 0)

        self.graphLevelComboBox = QComboBox(self)
        self.graphLevelComboBox.addItem("-3")
        self.graphLevelComboBox.addItem("-2")
        self.graphLevelComboBox.addItem("-1")
        self.graphLevelComboBox.addItem("0")
        self.graphLevelComboBox.addItem("1")
        self.graphLevelComboBox.addItem("2")
        self.graphLevelComboBox.addItem("3")
        self.graphLevelComboBox.addItem("4")
        self.graphLevelComboBox.addItem("5")
        self.graphLevelComboBox.addItem("6")
        self.graphLevelComboBox.setCurrentIndex(6)
        graphTabOptionsLayout.addWidget(self.graphLevelComboBox, 0, offset_opt + 1)
        self.graphLevelComboBox.currentIndexChanged.connect(self.showGraph)

        self.graphTypeComboBox = QComboBox(self)
        self.graphTypeComboBox.addItem("d - dot")
        self.graphTypeComboBox.addItem("c - circo")
        self.graphTypeComboBox.addItem("n - neato")
        self.graphTypeComboBox.addItem("f - fdp")
        self.graphTypeComboBox.setCurrentIndex(0)
        graphTabOptionsLayout.addWidget(QLabel("GraphViz engine:"), 0, offset_opt + 2)
        graphTabOptionsLayout.addWidget(self.graphTypeComboBox, 0, offset_opt + 3)
        self.graphTypeComboBox.currentIndexChanged.connect(self.showGraph)

        self.graphShowExtInputs = QCheckBox("Show ext inputs")
        self.graphShowExtInputs.setChecked(True)
        graphTabOptionsLayout.addWidget(self.graphShowExtInputs, 0, offset_opt + 4)
        self.graphShowExtInputs.toggled.connect(self.showGraph)

        self.graphShowInputPops = QCheckBox("Show input pops")
        self.graphShowInputPops.setChecked(True)
        graphTabOptionsLayout.addWidget(self.graphShowInputPops, 0, offset_opt + 5)
        self.graphShowInputPops.toggled.connect(self.showGraph)

        self.add_tab(self.LEMS_VIEW_TAB, self.tabs, image=True, options=True)

        self.add_tab(self.IMAGE_3D_TAB, self.tabs, image=True, options=True)

        self.tabs.addTab(self.matrixTab, "Matrix")
        self.matrixTabLayout = QGridLayout()
        self.matrixTab.setLayout(self.matrixTabLayout)

        # Add tabs
        self.tabs.addTab(self.nmlliteTab, "NeuroMLlite")

        self.nmlliteTabLayout = QGridLayout()
        self.nmlliteTab.setLayout(self.nmlliteTabLayout)

        self.nmlliteTabs = QTabWidget()
        self.nmlliteTabLayout.addWidget(self.nmlliteTabs)

        self.nmlliteSimTab = QWidget()
        self.nmlliteTabs.addTab(self.nmlliteSimTab, "Simulation")
        self.nmlliteSimTabLayout = QGridLayout()
        self.nmlliteSimTab.setLayout(self.nmlliteSimTabLayout)
        self.nmlliteSimText = QPlainTextEdit()
        self.nmlliteSimTabLayout.addWidget(self.nmlliteSimText, 0, 0)

        self.nmlliteNetTab = QWidget()
        self.nmlliteTabs.addTab(self.nmlliteNetTab, "Network")
        self.nmlliteNetTabLayout = QGridLayout()
        self.nmlliteNetTab.setLayout(self.nmlliteNetTabLayout)
        self.nmlliteNetText = QPlainTextEdit()
        self.nmlliteNetTabLayout.addWidget(self.nmlliteNetText, 0, 0)

        self.tabs.addTab(self.nml2Tab, "NeuroML 2")
        self.nml2Layout = QGridLayout()
        self.nml2Tab.setLayout(self.nml2Layout)
        self.nml2Text = QPlainTextEdit()
        self.nml2Text.insertPlainText(
            "\n   Generate an instance of the populations and projections in this network in NeuroML 2"
            + "\n   format by pressing the Generate NeuroML 2 button on the left"
            + "\n\n\n   WARNING: depending on the size of the generated network, text may take some time to load!"
        )
        self.nml2Layout.addWidget(self.nml2Text, 0, 0)

        # Add tabs to widget
        midLayout.addWidget(self.tabs, 0, 0)
        mainLayout = QGridLayout()

        topLayout.addWidget(nameLabel, 0, 0)
        # topLayout.addWidget(self.nameLine, 0, 1)

        rows = 0

        l = QLabel("Network parameters")
        l.setFont(header_font)
        paramLayout.addWidget(l, rows, 0)

        self.param_entries = {}

        if self.network.parameters is not None and len(self.network.parameters) > 0:
            for p in sorted(self.network.parameters.keys()):
                rows += 1
                pval = self.network.parameters[p]
                label = QLabel("%s" % p)
                paramLayout.addWidget(label, rows, 0)
                entry = self.get_value_entry(p, pval, self.param_entries)
                paramLayout.addWidget(entry, rows, 1)

        if self.network.seed is not None:
            rows += 1
            pval = self.network.seed
            label = QLabel("Net generation seed")
            paramLayout.addWidget(label, rows, 0)
            entry = self.get_value_entry("seed", pval, self.param_entries)
            paramLayout.addWidget(entry, rows, 1)

        if self.network.temperature is not None:
            rows += 1
            pval = self.network.temperature
            label = QLabel("Temperature")
            paramLayout.addWidget(label, rows, 0)
            entry = self.get_value_entry("temperature", pval, self.param_entries)
            paramLayout.addWidget(entry, rows, 1)

        self.graphButton = QPushButton("Generate graph")
        self.graphButton.show()
        self.graphButton.clicked.connect(self.showGraph)

        rows += 1
        paramLayout.addWidget(self.graphButton, rows, 0)

        self.lemsViewButton = QPushButton("Generate LEMS View")
        self.lemsViewButton.show()
        self.lemsViewButton.clicked.connect(self.showLemsView)

        rows += 1
        paramLayout.addWidget(self.lemsViewButton, rows, 0)

        self.image3DButton = QPushButton("Generate 3D image")
        self.image3DButton.show()
        self.image3DButton.clicked.connect(self.show3Dimage)

        rows += 1
        paramLayout.addWidget(self.image3DButton, rows, 0)

        self.matrixButton = QPushButton("Generate matrix")
        self.matrixButton.show()
        self.matrixButton.clicked.connect(self.showMatrix)

        rows += 1
        paramLayout.addWidget(self.matrixButton, rows, 0)

        self.nml2Button = QPushButton("Generate NeuroML 2")
        self.nml2Button.show()
        self.nml2Button.clicked.connect(self.generateNeuroML2)

        rows += 1
        paramLayout.addWidget(self.nml2Button, rows, 0)

        rows += 1
        l = QLabel("Simulation parameters")
        l.setFont(header_font)
        paramLayout.addWidget(l, rows, 0)

        self.sim_entries = {}
        svars = ["dt", "duration", "seed"]

        for s in svars:
            rows += 1
            sval = self.simulation.__getattribute__(s)
            if sval is not None:
                label = QLabel("%s" % s)
                paramLayout.addWidget(label, rows, 0)

                entry = self.get_value_entry(s, sval, self.sim_entries)
                paramLayout.addWidget(entry, rows, 1)

        rows += 1

        paramLayout.addWidget(QLabel("Simulator:"), rows, 0)

        self.simulatorComboBox = QComboBox(self)
        for sim in self.simulators:
            self.simulatorComboBox.addItem(sim)

        paramLayout.addWidget(self.simulatorComboBox, rows, 1)
        rows += 1

        self.runButton = QPushButton("Run simulation")
        self.runButton.show()
        self.runButton.clicked.connect(self.runSimulation)

        self.autoRunCheckBox = QCheckBox("Auto run")
        rows += 1
        paramLayout.addWidget(self.runButton, rows, 0)
        paramLayout.addWidget(self.autoRunCheckBox, rows, 1)

        mainLayout.addLayout(topLayout, 0, 1)
        mainLayout.addLayout(paramLayout, 1, 0)
        mainLayout.addLayout(midLayout, 1, 1)

        # self.setLayout(paramLayout)
        self.setLayout(mainLayout)

        self.setWindowTitle("NeuroMLlite GUI")

        self.update_network_json()
        self.update_simulation_json()

    def showMatrix(self):
        """Generate matrix buttom has been pressed"""

        print_v("Matrix button was clicked.")

        if len(self.network.projections) == 0:
            self.dialog_popup(
                "No projections present in network, and so no matrix to show"
            )
            return

        self.update_net_sim()
        self.tabs.setCurrentWidget(self.matrixTab)

        from neuromllite.MatrixHandler import MatrixHandler

        level = 2

        handler = MatrixHandler(level, nl_network=self.network)

        from neuromllite.NetworkGenerator import generate_network

        generate_network(self.network, handler, always_include_props=True, base_dir=".")

        print_("Done with MatrixHandler...", self.verbose)

    def generateNeuroML2(self):
        """Generate NeuroML 2 representation of network"""

        print_v("Generate NeuroML 2 button was clicked.")

        self.update_net_sim()
        from neuromllite.NetworkGenerator import generate_neuroml2_from_network

        nml_file_name, nml_doc = generate_neuroml2_from_network(
            self.network,
            print_summary=True,
            format="xml",
            base_dir=None,
            copy_included_elements=False,
            target_dir=None,
            validate=False,
            simulation=self.simulation,
        )

        with open(nml_file_name, "r") as reader:
            nml_txt = reader.read()

        self.nml2Text.clear()
        self.nml2Text.insertPlainText(nml_txt)

        self.tabs.setCurrentWidget(self.nml2Tab)

    def showLemsView(self):
        """Generate lemsView button has been pressed"""

        print_v("lemsView button was clicked.")

        self.update_net_sim()
        self.tabs.setCurrentWidget(self.all_tabs[self.LEMS_VIEW_TAB])
        self.update_net_sim()
        from neuromllite.NetworkGenerator import generate_neuroml2_from_network

        from neuromllite.NetworkGenerator import generate_and_run

        lems_file_name = generate_and_run(
            self.simulation,
            simulator="jNeuroML_norun",
            network=self.network,
            return_results=False,
            base_dir=self.sim_base_dir,
        )

        post_args = "-graph"

        from pyneuroml.pynml import run_jneuroml

        run_jneuroml("", lems_file_name, post_args, verbose=True)

        lems_view_file = lems_file_name.replace(".xml", ".png")

        self.add_image(lems_view_file, self.LEMS_VIEW_TAB)

    def show3Dimage(self):
        """Generate 3D view button has been pressed"""

        print_v("image3D button was clicked.")

        self.update_net_sim()
        self.tabs.setCurrentWidget(self.all_tabs[self.IMAGE_3D_TAB])

        from neuromllite.NetworkGenerator import generate_neuroml2_from_network

        nml_file_name, nml_doc = generate_neuroml2_from_network(
            self.network,
            print_summary=True,
            format="xml",
            base_dir=None,
            copy_included_elements=False,
            target_dir=None,
            validate=False,
            simulation=self.simulation,
        )

        post_args = "-png"

        from pyneuroml.pynml import run_jneuroml

        run_jneuroml("", nml_file_name, post_args, verbose=True)

        nml_view_file = nml_file_name.replace(".nml", ".png")

        self.add_image(nml_view_file, self.IMAGE_3D_TAB)

    def showGraph(self):
        """Generate graph buttom has been pressed"""

        print_v("Graph button was clicked.")

        self.update_net_sim()
        self.tabs.setCurrentWidget(self.all_tabs[self.GRAPH_TAB])

        from neuromllite.GraphVizHandler import GraphVizHandler

        engine = str(self.graphTypeComboBox.currentText()).split(" - ")[1]
        level = int(self.graphLevelComboBox.currentText())

        show_ext_inputs = self.graphShowExtInputs.isChecked()
        show_input_pops = self.graphShowInputPops.isChecked()

        format = "svg"
        format = "png"

        handler = GraphVizHandler(
            level,
            engine=engine,
            nl_network=self.network,
            output_format=format,
            view_on_render=False,
            include_ext_inputs=show_ext_inputs,
            include_input_pops=show_input_pops,
        )

        from neuromllite.NetworkGenerator import generate_network

        generate_network(self.network, handler, always_include_props=True, base_dir=".")

        print_("Done with GraphViz...", self.verbose)

        if format == "svg":
            genFile = "%s.gv.svg" % self.network.id

            self.add_image(genFile, self.GRAPH_TAB)
            """
            svgWidget = QSvgWidget(genFile)
            svgWidget.resize(svgWidget.sizeHint())
            svgWidget.show()
            self.graphTabLayout.addWidget(svgWidget, 0, 0)"""

        elif format == "png":
            genFile = "%s.gv.png" % self.network.id

            self.add_image(genFile, self.GRAPH_TAB)

    def update_net_sim(self):
        """Set the parameters in the network/simulation from the GUI values"""

        for p in self.param_entries:
            v = self.param_entries[p].text()
            try:
                v = int(v)
            except:
                try:
                    v = float(v)
                except:
                    pass  # leave as string...

            print_("Setting param %s to %s" % (p, v), self.verbose)
            if p == "seed":
                self.network.seed = v
            elif p == "temperature":
                self.network.temperature = v
            else:
                self.network.parameters[p] = v

        print_("All params: %s" % self.network.parameters, self.verbose)

        for s in self.sim_entries:
            v = float(self.sim_entries[s].text())
            print_("Setting simulation variable %s to %s" % (s, v), self.verbose)
            self.simulation.__setattr__(s, v)

    def runSimulation(self):
        """Run a simulation in the chosen simulator"""

        simulator = str(self.simulatorComboBox.currentText())
        print(
            "Run button was clicked. Running simulation %s in %s with %s"
            % (self.simulation.id, self.sim_base_dir, simulator)
        )

        self.tabs.setCurrentWidget(self.simTab)

        self.update_net_sim()

        from neuromllite.NetworkGenerator import generate_and_run

        # return
        try:
            self.current_traces, self.current_events = generate_and_run(
                self.simulation,
                simulator=simulator,
                network=self.network,
                return_results=True,
                base_dir=self.sim_base_dir,
            )

            self.replotSimResults()
        except Exception as e:
            self.dialog_popup("Error: %s" % e)

    def _get_sorted_population_ids(self, include_input_pops=True):
        all_pop_ids = []
        for pop in self.network.populations:
            if not include_input_pops and is_spiking_input_population(
                pop, self.network
            ):
                pass  # ignoring...
            else:
                all_pop_ids.append(pop.id)
        return sorted(all_pop_ids)

    def _get_pop_size(self, pop_id):
        pop = self.network.get_child(pop_id, "populations")
        return evaluate(pop.size, self.network.parameters)

    def _get_pop_id_cell_id(self, quantity):

        if "[" in quantity:
            # e.g. Epop[5]/v
            pop_id = quantity.split("[")[0]
            cell_id = int(quantity.split("[")[1].split("]")[0])
        else:
            # e.g. Epop/0/iafcell/v
            pop_id = quantity.split("/")[0]
            cell_id = int(quantity.split("/")[1])

        return pop_id, cell_id

    def traceSelect(self):

        print_v(
            "traceSelect button was clicked. Traces shown: %s; colours: %s"
            % (self.current_traces_shown, self.current_traces_colours)
        )

        dialog = QDialog(self)
        dialog.setWindowTitle("Select which traces to plot")

        QBtn = QDialogButtonBox.Ok  # | QDialogButtonBox.Cancel

        buttonBox = QDialogButtonBox(QBtn)
        buttonBox.accepted.connect(dialog.accept)

        layout = QGridLayout()
        dialog.setLayout(layout)

        count = 0
        self.all_cbs = {}

        for key in sorted(self.current_traces.keys()):
            if key != "t":
                self.all_cbs[key] = QCheckBox(key)
                self.all_cbs[key].setChecked(self.current_traces_shown[key])
                self.all_cbs[key].stateChanged.connect(
                    partial(self.traceSelectClicked, key)
                )
                color_button = QPushButton("%s" % self.current_traces_colours[key])
                style = (
                    "QPushButton {background-color: %s; color: black;}"
                    % self.current_traces_colours[key]
                )
                color_button.setStyleSheet(style)

                layout.addWidget(color_button, count, 0)
                layout.addWidget(self.all_cbs[key], count, 1)
                count += 1

        layout.addWidget(buttonBox, count, 1)
        dialog.exec_()
        self.replotSimResults()

    def traceSelectClicked(self, key):

        cb = self.all_cbs[key]
        # print('Clicked: %s, %s, key: %s'%(cb.text(),cb.isChecked(), key))
        self.current_traces_shown[key] = cb.isChecked()

    def _eval_at_all(self, expr, parameters, traces):
        tr_present = []
        for t in traces:
            if t != "t" and t in expr:
                tr_present.append(t)

        ret_val = []

        num_vals = len(traces[tr_present[0]])
        print_(
            "Evaluating %s with traces: %s (%i values) and params: %s"
            % (expr, tr_present, num_vals, parameters.keys()),
            self.verbose,
        )

        for i in range(num_vals):
            noo = expr
            for t in tr_present:
                noo = noo.replace(t, str(traces[tr_present[0]][i]))
                # print_v('%i: %s -> %s'%(i, expr, noo))

            r = evaluate(noo, parameters)
            ret_val.append(float(r))
        print_(
            "Generated: %s->%s (#%s)" % (ret_val[0], ret_val[-1], len(ret_val)),
            self.verbose,
        )
        return ret_val

    def replotSimResults(self):

        simulator = str(self.simulatorComboBox.currentText())
        self.traceSelectButton.setEnabled(True)

        info = "Data from sim of %s%s" % (
            self.simulation.id,
            " (%s)" % simulator if simulator else "",
        )

        pop_colors = {}

        for pop in self.network.populations:
            for prop in pop.properties:
                if prop == "color":
                    rgb = pop.properties[prop].split()
                    color = "#"
                    for a in rgb:
                        color = color + "%02x" % int(float(a) * 255)
                        pop_colors[pop.id] = color

        ## Plot traces

        xs = []
        ys = []
        labels = []
        colors = []
        colors_used = []
        heat_array = []

        for key in sorted(self.current_traces.keys()):

            if not key in self.current_traces_shown:
                self.current_traces_shown[key] = True

            if key != "t":
                heat_array.append(self.current_traces[key])
                pop_id = key.split("/")[0]

                if self.current_traces_shown[key]:
                    chosen_color = None

                    if key in self.current_traces_colours:
                        # print 'using stored for %s'%key
                        chosen_color = self.current_traces_colours[key]
                        colors.append(chosen_color)
                    else:
                        # print 'using new for %s'%key
                        if (
                            pop_id in pop_colors
                            and not pop_colors[pop_id] in colors_used
                        ):
                            chosen_color = pop_colors[pop_id]
                            colors_used.append(pop_colors[pop_id])
                        else:
                            if key in self.backup_colors:
                                chosen_color = self.backup_colors[key]
                            else:
                                chosen_color = get_next_hex_color()
                                self.backup_colors[key] = chosen_color

                        colors.append(chosen_color)
                        self.current_traces_colours[key] = chosen_color

                    xs.append(self.current_traces["t"])
                    ys.append(self.current_traces[key])
                    labels.append(key)

        ax_traces = self.tracesFigure.add_subplot(111)
        ax_traces.clear()

        for i in range(len(xs)):
            ax_traces.plot(
                xs[i], ys[i], label=labels[i], linewidth=0.5, color=colors[i]
            )

        ax_traces.set_xlabel("Time (s)")

        if self.showTracesLegend.isChecked():
            self.tracesFigure.legend()
        self.tracesCanvas.draw()

        ## Plot heatmap

        ax_heatmap = self.heatmapFigure.add_subplot(111)
        ax_heatmap.clear()

        if len(heat_array) > 0:
            cm = matplotlib.cm.get_cmap("jet")
            hm = ax_heatmap.pcolormesh(heat_array, cmap=cm)
            # cbar = ax_heatmap.colorbar(im)

            if self.heatmapColorbar == None:
                self.heatmapColorbar = self.heatmapFigure.colorbar(hm)
                self.heatmapColorbar.set_label("Firing rate")

            self.heatmapCanvas.draw()

        ## Plot 2D

        if self.simulation.plots2D is not None:

            for plot2D in self.simulation.plots2D:
                info = self.simulation.plots2D[plot2D]
                fig = self.all_figures[plot2D]

                ax_2d = fig.add_subplot(111)
                ax_2d.clear()

                x_axes = info["x_axis"]
                y_axes = info["y_axis"]

                if not type(x_axes) == dict:
                    x_axes = {plot2D: x_axes}
                    y_axes = {plot2D: y_axes}

                for a in x_axes:
                    x_axis = x_axes[a]
                    y_axis = y_axes[a]

                    ax_2d.set_xlabel(x_axis)
                    ax_2d.set_ylabel(y_axis)

                    print_(
                        "Plotting %s for %s in %s: %s" % (a, plot2D, fig, info),
                        self.verbose,
                    )

                    if x_axis in self.current_traces.keys():
                        xs = self.current_traces[x_axis]
                    else:
                        xs = self._eval_at_all(
                            x_axis, self.network.parameters, self.current_traces
                        )
                    if y_axis in self.current_traces.keys():
                        ys = self.current_traces[y_axis]
                    else:
                        ys = self._eval_at_all(
                            y_axis, self.network.parameters, self.current_traces
                        )

                    ax_2d.plot(xs, ys, linewidth=0.5, label=a)

                    from pyneuroml.analysis.NML2ChannelAnalysis import get_colour_hex

                    num_points = len(xs)
                    tot_dots = 2
                    if a != "a":
                        tot_dots = 2
                    for i in range(tot_dots):
                        fract = i / float(tot_dots - 1)

                        index = int(num_points * fract)
                        if index >= num_points:
                            index = -1
                        c = "%s" % get_colour_hex(fract, (0, 255, 0), (255, 0, 0))
                        # print('Point %s/%s, fract %s, index %i, %s'%(i,tot_dots, fract,index, c))
                        m = 4 if index == 0 or index == -1 else 2
                        ax_2d.plot(xs[index], ys[index], "o", color=c, markersize=m)
                fig.legend()

                self.all_canvases[plot2D].draw()

        ## Plot 3D

        if self.simulation.plots3D is not None:

            for plot3D in self.simulation.plots3D:
                info = self.simulation.plots3D[plot3D]
                fig = self.all_figures[plot3D]

                # ax_3d = fig.add_subplot(111)
                from mpl_toolkits.mplot3d import Axes3D

                ax_3d = fig.gca(projection="3d")

                ax_3d.clear()
                xs = self.current_traces[info["x_axis"]]
                ys = self.current_traces[info["y_axis"]]
                zs = self.current_traces[info["z_axis"]]

                num_points = len(xs)
                tot_dots = 300
                for i in range(tot_dots):
                    fract = i / float(tot_dots - 1)

                    index = int(num_points * fract)
                    if index >= num_points:
                        index = -1
                    c = "%s" % get_colour_hex(fract, (0, 255, 0), (255, 0, 0))
                    # print('Point %s/%s, fract %s, index %i, %s'%(i,tot_dots, fract,index, c))
                    m = 4 if index == 0 or index == -1 else 1.5
                    ax_3d.plot(
                        [xs[index]],
                        [ys[index]],
                        [zs[index]],
                        "o",
                        color=c,
                        markersize=m,
                    )

                ax_3d.plot(xs, ys, zs, linewidth=0.5)
                ax_3d.set_xlabel(info["x_axis"])
                ax_3d.set_ylabel(info["y_axis"])
                ax_3d.set_zlabel(info["z_axis"])

                print("Plotting for %s in %s: %s" % (plot3D, fig, info))

                self.all_canvases[plot3D].draw()

        ## Plot spikes

        spikesFigure = self.all_figures[self.SPIKES_RASTERPLOT]
        ax_spikes = spikesFigure.add_subplot(111)
        ax_spikes.clear()

        ids_for_pop = {}
        ts_for_pop = {}

        include_input_pops = self.rasterInPops.isChecked()
        pops_to_use = self._get_sorted_population_ids(
            include_input_pops=include_input_pops
        )

        for k in self.current_events.keys():
            spikes = self.current_events[k]

            pop_id, cell_id = self._get_pop_id_cell_id(k)
            if pop_id in pops_to_use:
                if not pop_id in ids_for_pop:
                    ids_for_pop[pop_id] = []
                    ts_for_pop[pop_id] = []

                for t in spikes:
                    ids_for_pop[pop_id].append(cell_id)
                    ts_for_pop[pop_id].append(t)

        max_id = 0

        for pop_id in sorted(ids_for_pop.keys()):

            if pop_id in pop_colors:
                c = pop_colors[pop_id]
            else:
                c = get_next_hex_color()

            if pop_id in ts_for_pop:
                shifted_ids = [id + max_id for id in ids_for_pop[pop_id]]
                ax_spikes.plot(
                    ts_for_pop[pop_id],
                    shifted_ids,
                    label=pop_id,
                    linewidth=0,
                    marker=".",
                    color=c,
                )

            pop = self.network.get_child(pop_id, "populations")
            if pop.size is not None:
                max_id_here = self._get_pop_size(pop_id)
            else:
                max_id_here = (
                    max(ids_for_pop[pop_id]) if len(ids_for_pop[pop_id]) > 0 else 0
                )

            print_(
                "Finished with spikes for %s, go from %i with max %i"
                % (pop_id, max_id, max_id_here),
                self.verbose,
            )
            max_id += max_id_here

        ax_spikes.set_xlabel("Time (s)")
        ax_spikes.set_ylabel("Index")

        tmax = self.simulation.duration / 1000.0
        ax_spikes.set_xlim(tmax / -20.0, tmax * 1.05)
        ax_spikes.set_ylim(max_id / -20.0, max_id * 1.05)

        if self.rasterLegend.isChecked():
            spikesFigure.legend()
        self.all_canvases[self.SPIKES_RASTERPLOT].draw()

        ## Plot pop spike rates

        popRateFigure = self.all_figures[self.SPIKES_POP_RATE_AVE]
        ax_pop_rate = popRateFigure.add_subplot(111)
        ax_pop_rate.clear()

        rates = {}
        print_("Generating rates from %s" % self.current_events.keys(), self.verbose)

        xs = []
        labels = []
        pop_sizes = {}
        count = 0
        rates = {}

        include_input_pops = self.spikeStatInPops.isChecked()
        pops_to_use = self._get_sorted_population_ids(
            include_input_pops=include_input_pops
        )

        for pop_id in pops_to_use:
            xs.append(count)
            labels.append(pop_id)
            count += 1
            pop_sizes[pop_id] = self._get_pop_size(pop_id)
            rates[pop_id] = []

        for k in self.current_events.keys():
            spikes = self.current_events[k]
            pop_id, cell_id = self._get_pop_id_cell_id(k)
            if pop_id in pops_to_use:
                rate = 1000 * len(spikes) / self.simulation.duration
                print_(
                    "%s: %s[%s] has %s spikes, so %s Hz"
                    % (k, pop_id, cell_id, len(spikes), rate),
                    self.verbose,
                )
                rates[pop_id].append(rate)

        avg_rates = []
        sd_rates = []
        import numpy as np

        for pop_id in pops_to_use:
            avg_rates.append(np.mean(rates[pop_id]) if len(rates[pop_id]) > 0 else 0)
            sd_rates.append(np.std(rates[pop_id]) if len(rates[pop_id]) > 0 else 0)

        print_v(
            "Calculated rates: %s; means: %s; stds: %s" % (rates, avg_rates, sd_rates)
        )

        bars = ax_pop_rate.bar(xs, avg_rates, yerr=sd_rates)

        ax_pop_rate.set_ylabel("Rate (Hz)")
        for bi in range(len(bars)):
            bar = bars[bi]
            bar.set_facecolor(pop_colors[labels[bi]])

        ax_pop_rate.set_xticks(xs)
        ax_pop_rate.set_xticklabels(labels)
        ax_pop_rate.set_xticklabels(
            ax_pop_rate.get_xticklabels(), rotation=45, horizontalalignment="right"
        )

        self.all_canvases[self.SPIKES_POP_RATE_AVE].draw()

        print_v("Finished plotting")


def main():
    """Main run method"""

    if len(sys.argv) == 1:
        usage()
        exit()

    app = QApplication(sys.argv)
    nml_sim_file = sys.argv[1]

    nmlui = NMLliteUI(nml_sim_file)
    nmlui.show()

    sys.exit(app.exec_())


def usage():

    from neuromllite import __version__ as version

    MAIN_CLA = "nmllite-ui"
    USAGE = """
NMLlite-UI v{0}: A GUI for loading NeuroMLlite files

Usage:
    {1} Sim_xxx.json
         Load a NeuroMLlite file containing a Simulation, which refers to the Network to run
    """.format(
        version, MAIN_CLA
    )
    print(USAGE)


if __name__ == "__main__":
    main()
